using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Wordprocessing;
using System.Data;
using System.Diagnostics;
using System.Reflection;
using System.Xml;
using System.Xml.Serialization;
using WordXmlMapSample.Data;
using WordXmlMapSample.Data.Entities;

namespace WordXmlMapSample
{
    public partial class Form1 : Form
    {
        const string TEMPLATE_PATH = "C:\\Temp\\Customer_Template.docx";
        const string DESTINATION_PATH = "C:\\Temp\\Customer_Filled.docx";

        private readonly WordTemplateHandler _handler;
        private readonly ApplicationDbContext _context;

        public Form1()
        {
            InitializeComponent();
            _handler = new();
            _context = new();
        }

        ~Form1()
        {
            if (_context is not null)
            {
                _context.Dispose();
            }
        }

        private void ButtonCreateCustomerWordTemplate_Click(object sender, EventArgs e)
        {
            using MemoryStream schemaStream = new();
            using XmlWriter writer = XmlWriter.Create(schemaStream, new() { Indent = false });
            XmlSerializer serializer = new(typeof(Customer), "xmlns:http://tempuri.org");
            serializer.Serialize(writer, new Customer() { Name = "Name", Address = "Address", City = "City", PostCode = "PostCode" });
            writer.Flush();
            schemaStream.Seek(0, SeekOrigin.Begin);

            _handler.CreateEmptyTemplate(TEMPLATE_PATH, schemaStream);
        }

        private void ButtonApplayTemplate_Click(object sender, EventArgs e)
        {
            using var context = new ApplicationDbContext();
            WordTemplate wordTemplate = context.WordTemplates.First();
            if (wordTemplate.Schema is null)
            {
                return;
            }
            using MemoryStream memoryStream = new(wordTemplate.Schema);
            using var wordDocument = WordprocessingDocument.Open(memoryStream, true);
            MainDocumentPart mainDocumentPart = wordDocument.MainDocumentPart!;
            mainDocumentPart.DeleteParts(mainDocumentPart.CustomXmlParts);

            Customer customer = context.Customers.First();

            using MemoryStream customerStream = new();
            using XmlWriter writer = XmlWriter.Create(customerStream, new() { Indent = false });
            XmlSerializer serializer = new(typeof(Customer), "xmlns:http://tempuri.org");
            serializer.Serialize(writer, customer);
            writer.Flush();
            customerStream.Seek(0, SeekOrigin.Begin);


            var customXmlPart = mainDocumentPart.AddCustomXmlPart(CustomXmlPartType.CustomXml);
            customXmlPart.FeedData(customerStream);

            using var n = wordDocument.Clone(DESTINATION_PATH);
            n.Save();
        }

        private void ButtonImportTemplate_Click(object sender, EventArgs e)
        {
            using FileStream fs = File.OpenRead(TEMPLATE_PATH);
            byte[] buffer = new byte[fs.Length];
            fs.Read(buffer, 0, buffer.Length);
            WordTemplate wordTemplate = new()
            {
                Name = "Customer Template",
                Schema = buffer
            };

            _context.WordTemplates.Add(wordTemplate);
            _context.SaveChanges();

            fs.Close();
        }
    }
}