﻿using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Wordprocessing;
using System.Text;

namespace WordXmlMapSample
{
    internal class WordTemplateHandler
    {
        public void CreateEmptyTemplate(string destination, string customXml)
        {
            if (string.IsNullOrWhiteSpace(destination))
            {
                throw new ArgumentNullException(nameof(destination));
            }

            if (string.IsNullOrWhiteSpace(customXml))
            {
                throw new ArgumentNullException(nameof(customXml));
            }

            using MemoryStream ms = new(Encoding.UTF8.GetBytes(customXml));
            DoCreateEmptyTemplate(destination, ms);
        }

        public void CreateEmptyTemplate(string destination, Stream customXmlStream)
        {
            if (string.IsNullOrWhiteSpace(destination))
            {
                throw new ArgumentNullException(nameof(destination));
            }

            if (customXmlStream is null)
            {
                throw new ArgumentNullException(nameof(customXmlStream));
            }

            DoCreateEmptyTemplate(destination, customXmlStream);
        }

        private void DoCreateEmptyTemplate(string destination, Stream customXmlStream)
        {
            using FileStream fs = File.Create(destination);
            using WordprocessingDocument wordDocument = WordprocessingDocument.Create(fs, DocumentFormat.OpenXml.WordprocessingDocumentType.Document);

            // Create empty Body
            MainDocumentPart mainDocumentPart = wordDocument.AddMainDocumentPart();
            mainDocumentPart.Document = new();
            mainDocumentPart.Document.AppendChild(new Body());

            CustomXmlPart customXmlPart = mainDocumentPart.AddCustomXmlPart(CustomXmlPartType.CustomXml);
            customXmlPart.FeedData(customXmlStream);

            wordDocument.Save();
        }
    }
}
